/*
 * Copyright 2000 by Jean-Claude Christophe <jch@oleane.net>
 *                                          <jch@bsdfr.org>
 *
 * Home page:  http://www.bsdfr.org/projects/tac_bsd/
 *
 * connect.c  This file contains functions for speak with the TACACS server
 */

#include <sys/types.h>
#include <sys/socket.h>
#include <sys/time.h>

#include <netinet/in.h>
#include <arpa/inet.h>
#include <string.h>
#include <netdb.h>
#include <stdio.h>
#include <stdlib.h>
#include <syslog.h>
#include <unistd.h>
#include <errno.h>

#include "tac_bsd.h"
#include "conf.h"

extern int errno;
int seq_no;        /* sequence of each packet we send */
int session_id;    /* id of the current session */

extern void connect_timeout(int);

int
send_data(pkt, pkt_len, sd)
    void *pkt;
    int  pkt_len;
    int  sd;
{
    fd_set sds;
    int rr;
    int try;
    struct timeval tv;
    
    FD_ZERO(&sds);
    FD_SET(sd, &sds);
    tv.tv_sec = 15;
    tv.tv_usec = 0;
    
    for (try = 0; try < 3; try++) {
	rr = select(sd + 1, NULL, &sds, NULL, &tv);
	if (FD_ISSET(sd, &sds)) {
	    if (pkt_len != write(sd, pkt, pkt_len))
		continue;
	    return(0);
	}
	return(-1);
    }
    return(-1);
}

int
read_data(pkt, pkt_len, sd)                 
    void *pkt;
    int   pkt_len;
    int   sd;
{                                                
    fd_set sds;
    int rr;
    int try;
    struct timeval tv;
    
    FD_ZERO(&sds);
    FD_SET(sd, &sds);
    tv.tv_sec = 15;
    tv.tv_usec = 0;
    
    for (try = 0; try < 3; try++) {
	rr = select(sd + 1, &sds, NULL, NULL, &tv);
	if (FD_ISSET(sd, &sds)) {
	    if (pkt_len != read(sd, pkt, pkt_len))
		continue;
	    return(0);
	}
	return(-1);
    }
    return(-1);
}

unsigned long
resolve_addr(host)
    char *host;
{
    struct in_addr addr;
    struct hostent *h;
 
    if (inet_aton(host, &addr) == 0) {
	if ((h = gethostbyname(host)) == NULL) {
	    herror(host);
	} else {
	    bcopy(h->h_addr, (char *)&addr, sizeof(struct in_addr));
	    return(addr.s_addr);
	}
    } else return(addr.s_addr);
    
    return(-1);
}

int
connect_to_srv(server)
    char *server;
{
    struct sockaddr_in serv_addr;
    struct servent *s;
    int sd, result, oldmask;
    char *server2 = strdup(server);
    char *port = (char *)malloc(5);

    if (!server) /* normally, server _CANNOT_ be NULL if we are here. */
	return(-1);

    bzero((char *) &serv_addr, sizeof(serv_addr));
    serv_addr.sin_family = AF_INET;
	
    port = strchr(server2, ':');
	
    if (port)
	serv_addr.sin_port = htons(atoi(port + 1)); /* +1 for remove the ':' */
    else {
	s = getservbyname("tacacs", "tcp");
	if (s == NULL)
	    serv_addr.sin_port = htons(DEFAULT_PORT);
	else {
	    serv_addr.sin_port = s->s_port;
	    endservent();
	}
    }
    
    serv_addr.sin_addr.s_addr = resolve_addr(strtok(server2, ":"));

    if (serv_addr.sin_addr.s_addr < 0)
	if (slog)
	    syslog(_syslog, "in %s, could not resolve %s", __FUNCTION__, \
		   server2);

    if ((sd = socket(AF_INET, SOCK_STREAM, 0)) < 0) {
	if (slog) 
	    syslog(_syslog, "in %s, socket %s:%d: %s", __FUNCTION__, \
		   server, ntohs(serv_addr.sin_port), strerror(errno));
    } else {
	if (server_timeout) {
	    oldmask = sigmask(SIGALRM);
	    (void)signal(SIGALRM, connect_timeout);
	    (void)alarm(server_timeout);
	}
	result = (connect(sd, (struct sockaddr *) &serv_addr, sizeof(serv_addr)));
	if (result < 0) {
	    if (slog) syslog(_syslog, "in %s, connect %s:%d: %s", __FUNCTION__, \
			     server2, ntohs(serv_addr.sin_port), strerror(errno));
	} else {
	    alarm(0);
	    return(sd);
	}
    }
    if (slog)
	syslog(_syslog, "in %s, The server is not online. Fatal.", __FUNCTION__);

    return(-1); /* error */
}

/* manage the timeout connection to the TACACS server */
void
connect_timeout(signo)
    int signo;
{
    fprintf(stderr, "%% TACACS: Server timeout after %d seconds\n", server_timeout);
}

